"""
1. Реализовать аутентификацию пользователей на сервере.
2. *Реализовать декоратор @login_required,
        проверяющий авторизованность пользователя для выполнения той или иной функции.
3. Реализовать хранение паролей в БД сервера
        (пароли не хранятся в открытом виде — хранится хэш-образ от пароля с добавлением криптографической соли).
4. *Реализовать возможность сквозного шифрования сообщений
        (использовать асимметричный шифр, ключи которого хранятся только у клиентов).
"""
# SERVER
import os
import sys
# import dis
# import select
import threading
import configparser
import log.server_log_config

# from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from common.globals import *
from common.utils import handle_parameters
# from time import time, localtime, strftime
from log.decorator import log
from server.server_db import ServerDB
from server.server_core import Core

from PyQt5.QtWidgets import QApplication, QMessageBox
# from PyQt5.QtCore import QTimer
from server.server_gui import MainWindow

# Инициализация логирования сервера.
LOGGER = logging.getLogger('server')

# Флаг, что был подключён новый пользователь, нужен чтобы не мучать BD
# постоянными запросами на обновление
new_connection = False
conflag_lock = threading.Lock()


def main():
    # Загрузка файла конфигурации сервера
    config = configparser.ConfigParser()

    dir_path = os.path.dirname(os.path.realpath(__file__)) + '/server'
    config.read(f"{dir_path}/{'server.ini'}")

    # Загрузка параметров командной строки, если нет параметров, то задаём
    # значения по умоланию.
    listen_ip, listen_port, _, _ = handle_parameters(
        config['SETTINGS']['listen_address'], config['SETTINGS']['default_port'])

    # Инициализация базы данных
    database = ServerDB(
        os.path.join(
            config['SETTINGS']['database_path'],
            config['SETTINGS']['database_file']))
    # client = ServerDB('client/server_base.db3')

    # server_app = QApplication(sys.argv)
    # main_window = MainWindow()

    # Создание экземпляра класса - сервера и его запуск:
    server = Core(database, listen_ip, listen_port)
    server.daemon = True
    server.start()

    # Создаём графическое окружение для сервера:
    server_qapp = QApplication(sys.argv)
    main_window = MainWindow(database, server, config)
    # server_app = QApplication(sys.argv)
    # main_window = MainWindow()



    # # Функция, обновляющая список подключённых, проверяет флаг подключения, и
    # # если надо обновляет список
    # def list_update():
    #     global new_connection
    #     if new_connection:
    #         main_window.active_clients_table.setModel(
    #             gui_create_model(database))
    #         main_window.active_clients_table.resizeColumnsToContents()
    #         main_window.active_clients_table.resizeRowsToContents()
    #         with conflag_lock:
    #             new_connection = False
    #
    # # Функция, создающая окно со статистикой клиентов
    # def show_all_users():
    #     global all_users_window
    #     all_users_window = AllUsersWindow()
    #     all_users_window.all_users_table.setModel(create_all_users_model(database))
    #     all_users_window.all_users_table.resizeColumnsToContents()
    #     all_users_window.all_users_table.resizeRowsToContents()
    #     all_users_window.show()
    #
    # # Функция, создающая окно со статистикой клиентов
    # def show_statistics():
    #     global stat_window
    #     stat_window = HistoryWindow()
    #     stat_window.history_table.setModel(create_stat_model(database))
    #     stat_window.history_table.resizeColumnsToContents()
    #     stat_window.history_table.resizeRowsToContents()
    #     stat_window.show()
    #
    # # Функция создающяя окно с настройками сервера.
    # def server_config():
    #     global config_window
    #     # Создаём окно и заносим в него текущие параметры
    #     config_window = ConfigWindow()
    #     config_window.db_path.insert(config['SETTINGS']['database_path'])
    #     config_window.db_file.insert(config['SETTINGS']['database_file'])
    #     config_window.port.insert(config['SETTINGS']['default_port'])
    #     config_window.ip.insert(config['SETTINGS']['listen_address'])
    #     config_window.save_btn.clicked.connect(save_server_config)
    #
    # # Функция сохранения настроек
    # def save_server_config():
    #     global config_window
    #     info_message = QMessageBox()
    #     config['SETTINGS']['database_path'] = config_window.db_path.text()
    #     config['SETTINGS']['database_file'] = config_window.db_file.text()
    #
    #     port = config_window.port.text()
    #     if is_port_bad(port):
    #         info_message.warning(config_window, 'Ошибка', f'Полученный PORT: {port} должен быть в пределах 1024-65535')
    #         return
    #     ip = config_window.ip.text()
    #     if is_ip_bad(ip):
    #         info_message.warning(config_window, 'Ошибка', f'Полученный IP: {ip} должен иметь формат: 127.0.0.1 ')
    #         return
    #
    #     # Если порт и ip в норме - сохраняем
    #     config['SETTINGS']['listen_address'] = ip
    #     config['SETTINGS']['default_port'] = port
    #     with open('server.ini', 'w') as conf:
    #         config.write(conf)
    #         info_message.information(
    #             config_window, 'OK', 'Настройки успешно сохранены!')

    # Таймер, обновляющий список клиентов 1 раз в 2 секунд


    # Запускаем GUI
    server_qapp.exec_()

    # print_help()
    #
    # while True:
    #     command = input('=>')
    #     if command.lower() in ['u', 'users']:
    #         for user in sorted(client.get_all_users_list()):
    #             print(f'Пользователь [{user[0]}], последний вход: {user[1]}')
    #     elif command.lower() in ['a', 'active']:
    #         for user in sorted(client.get_active_users_list()):
    #             print(f'Пользователь [{user[0]}] [{user[1]}:{user[2]}] вошел: {user[3]}')
    #     elif command.lower() in ['l', 'loghist']:
    #         name = input('Введите имя конкретного пользователя или нажмите Enter: ')
    #         for user in sorted(client.get_login_history(name)):
    #             print(f'Пользователь [{user[0]}], последний вход: {user[3]} с [{user[1]}:{user[2]}]')
    #     elif command.lower() in ['s', 'stat']:
    #         name = input('Введите имя конкретного пользователя или нажмите Enter: ')
    #         for data in sorted(client.get_user_stat(name)):
    #             print(f'Пользователь [{data[0]}], последний вход: {data[1]} '
    #                   f'сообщений отправлено: {data[2]} получено: {data[3]}]')
    #     elif command.lower() in ['?', 'help']:
    #         print_help()
    #     elif command.lower() in ['q', 'quit']:
    #         server.stop()
    #         break
    #     else:
    #         print(random.choice(
    #             ['Хватит лохматить бабушку, пиши help', 'Не тыкай!', 'Ананимус нашелся..',
    #              'Начинаю форматирование.....', 'Не твое - не трожь', 'Кому сказано, пиши help',
    #              'Только для взрослых', 'И кто тебя из клетки выпустил?']))


if __name__ == '__main__':
    main()



""""""
import logging
from sqlalchemy import create_engine, Column, Integer, String, DateTime, ForeignKey, Text
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session
from datetime import datetime

LOGGER = logging.getLogger('server')


class ServerDB:
    Base = declarative_base()

    # Список всех пользователей
    class AllUsers(Base):
        __tablename__ = 'all_users'
        id = Column(Integer, primary_key=True)
        user_name = Column(String, unique=True)
        last_login_time = Column(DateTime)
        pwd_hash = Column(String)
        pubkey = Column(Text)

        def __init__(self, user_name, pwd_hash):
            self.user_name = user_name
            self.last_login_time = datetime.now()
            self.pwd_hash = pwd_hash
            self.pubkey = None

    # Список активных пользователей
    class ActiveUsers(Base):
        __tablename__ = 'active_users'
        id = Column(Integer, primary_key=True)
        user_id = Column(ForeignKey('all_users.id'), unique=True)
        ip = Column(String)
        port = Column(Integer)
        login_time = Column(DateTime)

        def __init__(self, user_id, ip, port, login_time):
            self.user_id = user_id
            self.ip = ip
            self.port = port
            self.login_time = login_time

    # Список контактов каждого пользователя
    class UsersContacts(Base):
        __tablename__ = 'users_contacts'
        id = Column(Integer, primary_key=True)
        user_id = Column(ForeignKey('all_users.id'))
        contact_id = Column(ForeignKey('all_users.id'))

        def __init__(self, user_id, contact_id):
            self.user_id = user_id
            self.contact_id = contact_id

    # Список действий каждого пользователя
    class UsersStats(Base):
        __tablename__ = 'users_stats'
        id = Column(Integer, primary_key=True)
        user_id = Column(ForeignKey('all_users.id'), unique=True)
        sent = Column(Integer)
        received = Column(Integer)

        def __init__(self, user_id):
            self.user_id = user_id
            self.sent = 0
            self.received = 0

    # История входа пользователей
    class LoginHistory(Base):
        __tablename__ = 'login_history'
        id = Column(Integer, primary_key=True)
        user_id = Column(String, ForeignKey('all_users.id'))
        ip = Column(String)
        port = Column(Integer)
        last_login_time = Column(DateTime)

        def __init__(self, user_id, ip, port, last_login_time):
            self.user_id = user_id
            self.ip = ip
            self.port = port
            self.last_login_time = last_login_time

    # Инициализация
    def __init__(self, path):  # client/server_base.db3
        self.engine = create_engine(f'sqlite:///{path}?check_same_thread=False',
                                    echo=False, pool_recycle=7200, encoding='utf-8')
        self.Base.metadata.create_all(self.engine)

        Session = scoped_session(sessionmaker(bind=self.engine))
        self.session = Session()

        self.session.query(self.ActiveUsers).delete()
        self.session.commit()

    # Вход пользователя
    def user_login(self, user_name, ip, port, key):
        searchable_user = self.session.query(self.AllUsers).filter_by(user_name=user_name)
        if searchable_user.count():
            user = searchable_user.first()
            user.last_login_time = datetime.now()
            if user.pubkey != key:
                user.pubkey = key
        else:
            LOGGER.error(f'user_login: Пользователь {user_name} не зарегистрирован')
            return
            # user = self.AllUsers(user_name)
            # self.session.add(user)
            # self.session.commit()
            # new_user_stat = self.UsersStats(user.id)
            # self.session.add(new_user_stat)

        new_active_user = self.ActiveUsers(user.id, ip, port, datetime.now())
        self.session.add(new_active_user)
        user_history = self.LoginHistory(user.id, ip, port, datetime.now())
        self.session.add(user_history)
        self.session.commit()

    # Выход пользователя
    def user_logout(self, user_name):
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        if not user:
            LOGGER.error(f'user_logout: Пользователь {user_name} не найден в базе')
            return
        self.session.query(self.ActiveUsers).filter_by(user_id=user.id).delete()
        self.session.commit()

    # Получить список всех пользователей
    def get_all_users_list(self):
        query = self.session.query(self.AllUsers.user_name, self.AllUsers.last_login_time)
        return query.all()

    # Получить список активных пользователей
    def get_active_users_list(self):
        query = self.session.query(
            self.AllUsers.user_name,
            self.ActiveUsers.ip,
            self.ActiveUsers.port,
            self.ActiveUsers.login_time
        ).join(self.AllUsers)
        return query.all()

    def add_user(self, user_name, pwd_hash):
        """
        Метод регистрации пользователя.
        Принимает имя и хэш пароля, создаёт запись в таблице статистики.
        """
        user_row = self.AllUsers(user_name, pwd_hash)
        self.session.add(user_row)
        self.session.commit()
        history_row = self.UsersStats(user_row.id)
        self.session.add(history_row)
        self.session.commit()

    def remove_user(self, user_name):
        """Метод удаляющий пользователя из базы."""
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        self.session.query(self.ActiveUsers).filter_by(user_id=user.id).delete()
        self.session.query(self.LoginHistory).filter_by(user_id=user.id).delete()
        self.session.query(self.UsersContacts).filter_by(user_id=user.id).delete()
        self.session.query(
            self.UsersContacts).filter_by(
            contact_id=user.id).delete()
        self.session.query(self.UsersStats).filter_by(user_id=user.id).delete()
        self.session.query(self.AllUsers).filter_by(user_name=user_name).delete()
        self.session.commit()

    def get_hash(self, user_name):
        """Метод получения хэша пароля пользователя."""
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        return user.pwd_hash

    def get_pubkey(self, user_name):
        """Метод получения публичного ключа пользователя."""
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        return user.pubkey

    # Функция проверяет наличие пользователя в таблице Известных Пользователей
    def check_user(self, user_name):
        if self.session.query(self.AllUsers).filter_by(user_name=user_name).count():
            return True
        else:
            return False

    # Добавление пользователя в контакты
    def add_contact(self, user_name, contact_name):
        contact = self.session.query(self.AllUsers).filter_by(user_name=contact_name).first()
        if not contact:
            return
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        if not user:
            return
        if self.session.query(self.UsersContacts).filter_by(user_id=user.id, contact_id=contact.id).count():
            return
        new_contact = self.UsersContacts(user.id, contact.id)
        self.session.add(new_contact)
        self.session.commit()
        return True

    # Удаляем пользователя из контактов
    def del_contact(self, user_name, contact_name):
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        if not user:
            return
        contact = self.session.query(self.AllUsers).filter_by(user_name=contact_name).first()
        if not contact:
            return
        self.session.query(self.UsersContacts).filter_by(user_id=user.id, contact_id=contact.id).delete()
        self.session.commit()
        return True

    # Получить список контактов пользователя
    def get_contacts(self, user_name):
        user = self.session.query(self.AllUsers).filter_by(user_name=user_name).first()
        if not user:
            return
        contact_list = self.session.query(self.UsersContacts, self.AllUsers.user_name). \
            filter_by(user_id=user.id). \
            join(self.AllUsers, self.UsersContacts.contact_id == self.AllUsers.id)
        return [contact_name[1] for contact_name in contact_list.all()]

    # Получить историю входа пользователей
    def get_login_history(self, user_name=None):
        query = self.session.query(
            self.AllUsers.user_name,
            self.LoginHistory.ip,
            self.LoginHistory.port,
            self.LoginHistory.last_login_time
        ).join(self.AllUsers)
        if user_name:
            query = query.filter(self.AllUsers.user_name == user_name)
        return query.all()

    # Функция возвращает количество переданных и полученных сообщений
    def get_user_stat(self, user_name=None):
        query = self.session.query(
            self.AllUsers.user_name,
            self.AllUsers.last_login_time,
            self.UsersStats.sent,
            self.UsersStats.received
        ).join(self.AllUsers)
        if user_name:
            query = query.filter(self.AllUsers.user_name == user_name)
        # Возвращаем список кортежей
        return query.all()

    # Функция фиксирует передачу сообщения и делает соответствующие отметки в БД
    def process_message(self, sender, recipient):
        # Получаем ID отправителя и получателя
        sender = self.session.query(self.AllUsers).filter_by(user_name=sender).first().id
        # Запрашиваем строки из истории и увеличиваем счётчики
        sender_row = self.session.query(self.UsersStats).filter_by(user_id=sender).first()
        sender_row.sent += 1

        # Если указан получатель, иначе для всех активных
        if recipient:
            recipient = self.session.query(self.AllUsers).filter_by(user_name=recipient).first().id
            recipient_row = self.session.query(self.UsersStats).filter_by(user_id=recipient).first()
            recipient_row.received += 1
        else:
            for recipient in self.session.query(self.ActiveUsers).all():
                recipient_row = self.session.query(self.UsersStats).filter_by(user_id=recipient.user_id).first()
                recipient_row.received += 1
        self.session.commit()


# Тест работы
if __name__ == '__main__':
    db = ServerDB('client/server_base.db3')
    db.user_login('Василий', '192.168.1.4', 8888)
    db.user_login('Анатолий', '192.168.1.5', 7778)
    print(f'{db.get_all_users_list()=}')
    # выводим список кортежей - активных пользователей
    print(f'{db.get_active_users_list()=}')
    # выполянем 'отключение' пользователя
    db.user_logout('Василий')
    print('db.user_logout("Василий")')
    print(f'{db.get_all_users_list()=}')
    # выполянем 'отключение' пользователя
    db.add_contact('Василий', 'Анатолий')
    print(f'{db.get_contacts("Василий")=}')
    db.del_contact('Василий', 'Анатолий')
    print(f'{db.get_contacts("Василий")=}')
    # выводим список активных пользователей
    print(f'{db.get_active_users_list()=}')
    db.user_logout('Анатолий')
    print('db.user_logout("Анатолий")')
    print(f'{db.get_all_users_list()=}')
    print(f'{db.get_active_users_list()=}')



import hashlib
import binascii
import logging
import threading
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QLabel, QTableView, QDialog, QPushButton, \
    QLineEdit, QFileDialog, QMessageBox, QMenu
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt, QEvent, QTimer
from common.utils import is_port_bad, is_ip_bad


LOGGER = logging.getLogger('server')
database_lock = threading.Lock()


# GUI - Создание таблицы QModel, для отображения в окне программы.
def create_all_users_model(database):
    list_users = database.get_all_users_list()
    list_table = QStandardItemModel()
    list_table.setHorizontalHeaderLabels(['Имя Клиента', 'Время последнего входа'])
    for row in list_users:
        user, time = row
        user = QStandardItem(user)
        user.setEditable(False)
        # Уберём миллисекунды из строки времени, т.к. такая точность не требуется.
        time = QStandardItem(str(time.replace(microsecond=0)))
        time.setEditable(False)
        list_table.appendRow([user, time])
    return list_table


# GUI - Функция реализующая заполнение таблицы историей сообщений.
def create_stat_model(database):
    # Список записей из базы
    hist_list = database.get_user_stat()

    # Объект модели данных:
    list_table = QStandardItemModel()
    list_table.setHorizontalHeaderLabels(
        ['Имя Клиента', 'Последний раз входил', 'Сообщений отправлено', 'Сообщений получено'])
    for row in hist_list:
        user, last_seen, sent, recvd = row
        user = QStandardItem(user)
        user.setEditable(False)
        last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
        last_seen.setEditable(False)
        sent = QStandardItem(str(sent))
        sent.setEditable(False)
        recvd = QStandardItem(str(recvd))
        recvd.setEditable(False)
        list_table.appendRow([user, last_seen, sent, recvd])
    return list_table


# Класс основного окна
class MainWindow(QMainWindow):
    def __init__(self, database, server_app, config):
        super().__init__()

        self.database = database
        self.server_thread = server_app
        self.config = config
        self.new_connection = False

        self.init_ui()

    def init_ui(self):
        # Кнопка выхода
        self.exitAction = QAction('Выход', self)
        self.exitAction.setShortcut('Ctrl+Q')
        self.exitAction.triggered.connect(qApp.quit)

        # Кнопка обновить список клиентов
        self.all_users_button = QAction('Все пользователи', self)

        # Кнопка вывести историю сообщений
        self.show_history_button = QAction('История клиентов', self)

        # Кнопка регистрации пользователя
        self.register_btn = QAction('Регистрация пользователя', self)

        # Кнопка настроек сервера
        self.config_btn = QAction('Настройки сервера', self)

        # Статусбар
        # dock widget
        self.statusBar()

        # Тулбар
        self.toolbar = self.addToolBar('MainBar')
        self.toolbar.addAction(self.exitAction)
        self.toolbar.addAction(self.all_users_button)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.register_btn)
        self.toolbar.addAction(self.config_btn)

        # Настройки геометрии основного окна
        # Поскольку работать с динамическими размерами мы не умеем, и мало времени на изучение, размер окна фиксирован.
        self.setFixedSize(800, 600)
        self.setWindowTitle('Messaging Server alpha release')
        self.statusBar().showMessage('Server Working')

        # Надпись о том, что ниже список подключённых клиентов
        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(400, 15)
        self.label.move(10, 35)

        # Окно со списком подключённых клиентов.
        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 55)
        self.active_clients_table.setFixedSize(780, 400)

        # Инициализируем обновления таблицы
        self.timer = QTimer()
        self.timer.timeout.connect(self.create_online_model)
        self.timer.start(2000)

        # Связываем кнопки с процедурами
        self.all_users_button.triggered.connect(self.show_all_users)
        self.show_history_button.triggered.connect(self.show_statistics)
        self.register_btn.triggered.connect(self.reg_user)
        self.config_btn.triggered.connect(self.server_config)

        # Последним параметром отображаем окно.
        self.show()

    # Функция, обновляющая список подключённых, проверяет флаг подключения, и
    # если надо обновляет список
    # GUI - Создание таблицы QModel, для отображения в окне программы.
    def create_online_model(self):
        with database_lock:
            list_users = self.database.get_active_users_list()
        list_table = QStandardItemModel()
        list_table.setHorizontalHeaderLabels(['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
        for row in list_users:
            user, ip, port, time = row
            user = QStandardItem(user)
            user.setEditable(False)
            ip = QStandardItem(ip)
            ip.setEditable(False)
            port = QStandardItem(str(port))
            port.setEditable(False)
            # Уберём миллисекунды из строки времени, т.к. такая точность не требуется.
            time = QStandardItem(str(time.replace(microsecond=0)))
            time.setEditable(False)
            list_table.appendRow([user, ip, port, time])
        self.active_clients_table.setModel(list_table)
        self.active_clients_table.resizeColumnsToContents()
        self.active_clients_table.resizeRowsToContents()

    # Функция, создающая окно со статистикой клиентов
    def show_statistics(self):
        global stat_window
        stat_window = HistoryWindow()
        stat_window.history_table.setModel(create_stat_model(self.database))
        stat_window.history_table.resizeColumnsToContents()
        stat_window.history_table.resizeRowsToContents()
        # stat_window.show()

    # Функция, создающая окно со статистикой клиентов
    def show_all_users(self):
        global all_users_window
        all_users_window = AllUsersWindow(self.database, self.server_thread)
        all_users_window.all_users_table.setModel(create_all_users_model(self.database))
        all_users_window.all_users_table.resizeColumnsToContents()
        all_users_window.all_users_table.resizeRowsToContents()
        # all_users_window.show()

    # Функция создающяя окно с настройками сервера.
    def server_config(self):
        global config_window
        # Создаём окно и заносим в него текущие параметры
        config_window = ConfigWindow()
        config_window.db_path.insert(self.config['SETTINGS']['database_path'])
        config_window.db_file.insert(self.config['SETTINGS']['database_file'])
        config_window.port.insert(self.config['SETTINGS']['default_port'])
        config_window.ip.insert(self.config['SETTINGS']['listen_address'])
        config_window.save_btn.clicked.connect(config_window.save_server_config)
        # config_window.show()

    def reg_user(self):
        '''Метод создающий окно регистрации пользователя.'''
        global reg_window
        reg_window = RegisterUser(self.database, self.server_thread)
        reg_window.show()


# Класс окна с историей пользователей
class AllUsersWindow(QDialog):
    def __init__(self, database, server):
        super().__init__()
        self.database = database
        self.server = server
        # print(f'{self.server=}')
        self.initUI()

    def initUI(self):
        # Настройки окна:
        self.setWindowTitle('Список всех клиентов')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнопка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно историей
        self.all_users_table = QTableView(self)
        self.all_users_table.move(10, 10)
        self.all_users_table.setFixedSize(580, 620)
        self.all_users_table.installEventFilter(self)
        self.all_users_table.setSelectionBehavior(1)#setSelectionMode(1)
        # self.all_users_table.clicked.connect(self.delete_user)
        # itemDelegateForRow().mousePressEvent(QMouseEvent=C)

        self.show()

    def eventFilter(self, source, event):
        # if event.type() == QEvent.MouseButtonPress:
        #     print(event.button())
        #     print(event.pos())
        #     print(f'{self.itemAt(event.pos())=}')
        if event.type() == QEvent.ContextMenu:
            if source is self.all_users_table:
                # pos = event.pos()
                # row = source.selectedIndexes()#serowAt(event.pos().y())

                # print(f'{self.itemAt(event.pos())=}')
                # print(pos)
                # user = source.indexAt(event.pos())
                # print(f'{user.data()=}')

                try:
                    row = source.selectedIndexes()
                    if row:
                        user = str(row[0].data())
                        menu = QMenu()
                        menu.addAction('Удалить пользователя')
                        if menu.exec_(event.globalPos()):
                            # print(f'{user=}')
                            # if user in self.server.clients:
                            print(f'deleting: {user=}')
                            global del_popup
                            del_popup = DelUserDialog(self.database, self.server, user)
                        return True
                except Exception as e:
                    LOGGER.error(f'Не удалось удалить пользователя: {e}')
        return super().eventFilter(source, event)


class DelUserDialog(QDialog):
    def __init__(self, database, server, user):
        super().__init__()
        self.database = database
        self.server = server
        self.user = user

        self.setFixedSize(350, 120)
        self.setWindowTitle(f'Удаление пользователя: {user}')
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel(
            f'ВНИМАНИЕ!\nПользователь {self.user} будет УДАЛЕН из базы.\nПодтвердите действие.', self)
        self.selector_label.setFixedSize(330, 50)
        self.selector_label.move(10, 0)

        self.btn_ok = QPushButton('Удалить', self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(50, 70)
        self.btn_ok.clicked.connect(self.remove_user)

        self.btn_cancel = QPushButton('Отмена', self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(200, 70)
        self.btn_cancel.clicked.connect(self.close)

        self.show()

    #     self.all_users_fill()
    #
    # def all_users_fill(self):
    #     '''Метод заполняющий список пользователей.'''
    #     self.selector.addItems([item[0]
    #                             for item in self.database.users_list()])

    def remove_user(self):
        '''Метод - обработчик удаления пользователя.'''
        LOGGER.debug(f'Удаляю пользователя: {self.user}')
        self.server.remove_client(self.user)
        with database_lock:
            self.database.remove_user(self.user)
        # LOGGER.debug(f'Удаляю пользователя2: {self.user}')
        # if self.user in self.server.clients:
        #     LOGGER.debug(f'Удаляю пользователя: {self.user}')
        #     sock = self.server.clients[self.user]
        #     del self.server.clients[self.user]
        #     self.server.remove_client(sock)
        LOGGER.debug(f'Удален. Обновляю списки у клиентов...')
        # Рассылаем клиентам сообщение о необходимости обновить справочники
        self.server.service_update_lists()
        self.close()


# Класс окна с историей пользователей
class HistoryWindow(QDialog):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Настройки окна:
        self.setWindowTitle('Статистика клиентов')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнопка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно историей
        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.show()


# Класс окна настроек
class ConfigWindow(QDialog):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Настройки окна
        self.setFixedSize(365, 260)
        self.setWindowTitle('Настройки сервера')

        # Надпись о файле базы данных:
        self.db_path_label = QLabel('Путь до файла базы данных: ', self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        # Строка с путём базы
        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        # Кнопка выбора пути.
        self.db_path_select = QPushButton('Обзор...', self)
        self.db_path_select.move(275, 28)

        # Функция обработчик открытия окна выбора папки
        def open_file_dialog():
            global dialog
            dialog = QFileDialog(self)
            path = dialog.getExistingDirectory()
            path = path.replace('/', '\\')
            self.db_path.insert(path)

        self.db_path_select.clicked.connect(open_file_dialog)

        # Метка с именем поля файла базы данных
        self.db_file_label = QLabel('Имя файла базы данных: ', self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        # Поле для ввода имени файла
        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150, 20)

        # Метка с номером порта
        self.port_label = QLabel('Номер порта для соединений:', self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        # Поле для ввода номера порта
        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        # Метка с адресом для соединений
        self.ip_label = QLabel('С какого IP принимаем соединения:', self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        # Метка с напоминанием о пустом поле.
        self.ip_label_note = QLabel(' оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.', self)
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        # Поле для ввода ip
        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        # Кнопка сохранения настроек
        self.save_btn = QPushButton('Сохранить', self)
        self.save_btn.move(190, 220)

        # Кнопка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.show()

    # Функция сохранения настроек
    def save_server_config(self):
        global config_window
        info_message = QMessageBox()
        self.config['SETTINGS']['database_path'] = self.db_path.text()
        self.config['SETTINGS']['database_file'] = self.db_file.text()

        port = self.port.text()
        if is_port_bad(port):
            info_message.warning(self, 'Ошибка', f'Полученный PORT: {port} должен быть в пределах 1024-65535')
            return
        ip = self.ip.text()
        if is_ip_bad(ip):
            info_message.warning(self, 'Ошибка', f'Полученный IP: {ip} должен иметь формат: 127.0.0.1 ')
            return

        # Если порт и ip в норме - сохраняем
        self.config['SETTINGS']['listen_address'] = ip
        self.config['SETTINGS']['default_port'] = port
        with open('../../../lesson_05/server/server.ini', 'w') as conf:
            self.config.write(conf)
            info_message.information(
                self, 'OK', 'Настройки успешно сохранены!')


class RegisterUser(QDialog):
    """ Класс диалог регистрации пользователя на сервере. """

    def __init__(self, database, server):
        super().__init__()

        self.database = database
        self.server = server

        self.setWindowTitle('Регистрация')
        self.setFixedSize(175, 183)
        self.setModal(True)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.label_username = QLabel('Введите имя пользователя:', self)
        self.label_username.move(10, 10)
        self.label_username.setFixedSize(150, 15)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)

        self.label_passwd = QLabel('Введите пароль:', self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)

        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)
        self.label_conf = QLabel('Введите подтверждение:', self)
        self.label_conf.move(10, 100)
        self.label_conf.setFixedSize(150, 15)

        self.client_conf = QLineEdit(self)
        self.client_conf.setFixedSize(154, 20)
        self.client_conf.move(10, 120)
        self.client_conf.setEchoMode(QLineEdit.Password)

        self.btn_ok = QPushButton('Сохранить', self)
        self.btn_ok.move(10, 150)
        self.btn_ok.clicked.connect(self.save_data)

        self.btn_cancel = QPushButton('Выход', self)
        self.btn_cancel.move(90, 150)
        self.btn_cancel.clicked.connect(self.close)

        self.messages = QMessageBox()

        self.show()

    def save_data(self):
        """
        Метод проверки правильности ввода и сохранения в базу нового пользователя.
        """
        if not self.client_name.text():
            self.messages.critical(
                self, 'Ошибка', 'Не указано имя пользователя.')
            return
        elif self.client_passwd.text() != self.client_conf.text():
            self.messages.critical(
                self, 'Ошибка', 'Введённые пароли не совпадают.')
            return
        elif self.check_user(self.client_name.text()):
            self.messages.critical(
                self, 'Ошибка', 'Пользователь уже существует.')
            return
        else:
            # Генерируем хэш пароля, в качестве соли будем использовать логин в
            # нижнем регистре.
            passwd_bytes = self.client_passwd.text().encode('utf-8')
            salt = self.client_name.text().lower().encode('utf-8')
            passwd_hash = hashlib.pbkdf2_hmac(
                'sha512', passwd_bytes, salt, 10000)
            self.database.add_user(
                self.client_name.text(),
                binascii.hexlify(passwd_hash))
            self.messages.information(
                self, 'Успех', 'Пользователь успешно зарегистрирован.')
            # Рассылаем клиентам сообщение о необходимости обновить справочники
            self.server.service_update_lists()
            self.close()

    def check_user(self, name):
        with database_lock:
            return self.database.check_user(name)
# if __name__ == '__main__':
#     app = QApplication(sys.argv)
#     main_window = MainWindow()
#     main_window.statusBar().showMessage('Test Statusbar Message')
#     test_list = QStandardItemModel(main_window)
#     test_list.setHorizontalHeaderLabels(['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
#     test_list.appendRow(
#         [QStandardItem('test1'), QStandardItem('192.198.0.5'), QStandardItem('23544'), QStandardItem('16:20:34')])
#     test_list.appendRow(
#         [QStandardItem('test2'), QStandardItem('192.198.0.8'), QStandardItem('33245'), QStandardItem('16:22:11')])
#     main_window.active_clients_table.setModel(test_list)
#     main_window.active_clients_table.resizeColumnsToContents()
#     app.exec_()

    # ----------------------------------------------------------
    # app = QApplication(sys.argv)
    # window = HistoryWindow()
    # test_list = QStandardItemModel(window)
    # test_list.setHorizontalHeaderLabels(
    #     ['Имя Клиента', 'Последний раз входил', 'Отправлено', 'Получено'])
    # test_list.appendRow(
    #     [QStandardItem('test1'), QStandardItem('Fri Dec 12 16:20:34 2020'), QStandardItem('2'), QStandardItem('3')])
    # test_list.appendRow(
    #     [QStandardItem('test2'), QStandardItem('Fri Dec 12 16:23:12 2020'), QStandardItem('8'), QStandardItem('5')])
    # window.history_table.setModel(test_list)
    # window.history_table.resizeColumnsToContents()
    #
    # app.exec_()

    # ----------------------------------------------------------
    # app = QApplication(sys.argv)
    # dial = ConfigWindow()
    #
    # app.exec_()



from setuptools import setup, find_packages

setup(name="pyqt_hyperchat_server",
      version="0.1.8",
      description="pyqt_hyperchat_server",
      author="Wrexan",
      author_email="fwrexan@gmail.com",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



"""Лаунчер. Если не работает, запускайте через start_server.bat и start_client_***.bat
Для корректного подключения клиентов на сервере дожны быть зарегистрированы пользователи:
test1, test1, test3 с паролем 123456
"""
import subprocess
from time import sleep


def main(clients):
    process = []

    while True:
        action = input('Выберите действие: q - выход, s - запустить сервер и клиенты, x - закрыть все окна: ')

        if action == 'q':
            break
        elif action == 's':

            # Запуск сервера
            process.append(subprocess.Popen('python start_server.py', creationflags=subprocess.CREATE_NEW_CONSOLE))
            sleep(1)

            # Запуск клиентов
            for i in range(clients):
                process.append(subprocess.Popen(f'python start_client.py '
                                                f'-a 127.0.0.1 '
                                                f'-p 7777 '
                                                f'-name test{i+1} '
                                                f'-pwd 123456',
                                                creationflags=subprocess.CREATE_NEW_CONSOLE))
        elif action == 'x':
            while process:
                process.pop().kill()


if __name__ == '__main__':
    main(3)
